%define debug_package %{nil}
%define _build_id_links alldebug
%global daemon_user magellan
%define member_groups apache,monitor
%global data_dir %{_localstatedir}/opt/magellan
%global backup_modules %{_sysconfdir}/op5-backup/modules/external
%define pkg_sitelib %(echo %python3_sitelib | sed -e 's|/usr|%{prefix}|')
%define pkg_sitearch %(echo %python3_sitearch | sed -e 's|/usr|%{prefix}|')
Name: magellan
Version: %{op5version}
Release: %{op5release}
License: op5 Software License
Vendor: OP5 AB
Url: https://www.itrsgroup.com
BuildRoot: %{_tmppath}/%{name}-%{version}
Summary: Autodiscovery service
Group: Applications/Internet
Prefix: /opt/magellan
Requires: op5-lmd
Requires: mariadb-server
Requires: python3
Requires: fping >= 5.0
Requires: op5-monitor-user > 2021.2.1-op5.99
%systemd_requires
BuildRequires: m4
BuildRequires: python3-devel
Source: %{name}-%{version}.tar.gz
# This package has changed from noarch to arch specific. Therefor obsolete
# everything before the last released noarch package (v2021.5.3), to avoid yum
# "Error: Protected multilib versions".
Obsoletes: %{name} <= 2021.5.3-op5.2

%description
Network Autodiscovery Service for ITRS OP5 Monitor.

%package test
Summary: Test files for Magellan
Requires: magellan == %version
Requires: op5backup

%description test
Test tools to run feature test suite for Magellan.

%prep
%setup -q


%build
for src in op5build/*.m4
do
	m4 -D PREFIX_DIR=%{prefix} -D DATA_DIR=%{data_dir} -D SERVICE_USER=%{daemon_user} \
		-D PYTHON=%{__python3} \
		-D PYTHON_SITELIB=%pkg_sitelib \
		-D PYTHON_SITEARCH=%pkg_sitearch \
		$src > ${src%.m4}
done

%install
%{__install} -D -m 750 create-db.sh %{buildroot}%{prefix}/create-db.sh
%{__install} -D -m 644 -t %{buildroot}%{prefix} api-definition.* magellan.conf
%{__install} -D -m 644 op5build/magellan.service %{buildroot}%{_unitdir}/magellan.service
%{__install} -D -m 644 op5build/sysconfig %{buildroot}%{_sysconfdir}/sysconfig/magellan
%{__install} -D -m 644 op5build/op5-backup %{buildroot}%{backup_modules}/op5-magellan
%{__install} -D op5build/entrypoint.sh %{buildroot}%{prefix}/bin/magellan
%{__install} --directory %{buildroot}%{data_dir}

# Install Python binary packages built in pre-build step:

# Upgrade to pip version used for building the wheels in pre-build step
%{__python3} -m pip install --user --no-index -f dist/ --upgrade pip

# Install magellan and its dependencies
%{__python3} -m pip install -I --no-index --no-warn-script-location --no-deps \
	--prefix %{prefix} --root %{buildroot} dist/*.whl

# Install test tools for magellan-test package
%{__python3} -m pip install -I --no-index --no-warn-script-location --no-deps \
	--prefix %{prefix}/test --root %{buildroot} test-wheels/*.whl


%pre
getent group %daemon_user >/dev/null || groupadd -f -r %daemon_user
getent passwd %daemon_user >/dev/null || \
	useradd -g %daemon_user -r -M -N -s /sbin/nologin -d %data_dir \
	-c "OP5 Autodiscovery service" %daemon_user || :

# Add magellan to the group lmd runs under, for read access to it's unix socket.
usermod -a -G %member_groups %daemon_user || :

%post
# For upgrades from < 2021.6 which had Python virtual environment, make sure
# magellan is stopped before we can remove the venv directory.
if [ -e %prefix/venv ] && systemctl is-active --quiet magellan; then
	# MON-12726 workaround: Before v2021.5.1 magellan doesn't fully stop on _one_ SIGTERM.
	# Therefore first instruct systemd to stop magellan, but don't block command;
	systemctl stop --no-block magellan >/dev/null 2>&1 || :
	# ..courtesy wait to let magellan react to SIGTERM..
	pgrep magellan-server >/dev/null && sleep 3
	# ..then send another SIGTERM, which should make magellan shut down all its
	# threads (no-op if service is dead already).
	systemctl kill -s SIGTERM magellan >/dev/null 2>&1 || :
	# then issue stop command again, which blocks until service is stopped
	# (no-op if service is dead already).
	systemctl stop magellan >/dev/null 2>&1 || :
fi

# Remove any virtual environment from previous versions.
%{__rm} -rf %prefix/venv || :

if [ $1 -eq 2 ]; then
	# Remove DB user 'magellan'@'%' created in previous versions if it still exists.
	if systemctl -q is-active mariadb.service; then
		if mysql --no-defaults -B -N -u root -e \
				"SELECT 'exists' FROM mysql.user WHERE User = 'magellan' AND Host = '%'" | grep -q exists; then
			mysql --no-defaults -u root -e "DROP USER 'magellan'@'%'" || :
		fi
	fi

	# Move data from old to new data dir on upgrade, but only if source dir has
	# data and target dir is empty.
	if [ -d %prefix/data ] && ls -q %prefix/data | grep -q "."; then
		if ! ls -qA %data_dir | grep -q "."; then
			for f in %{prefix}/data/*; do
				%__mv "$f" %data_dir/
			done
			%__chown -R %daemon_user:%daemon_user %data_dir/*
			%__chmod -R go-w %data_dir/*
			%__rm -rf %prefix/data
		else
			echo "Warning: Skip moving data from old location '%prefix/data' since new location '%data_dir' is not empty."
		fi
	fi
fi

systemctl daemon-reload >/dev/null 2>&1 || :
if [ $1 -eq 2 ]; then
	systemctl restart magellan >/dev/null 2>&1 || :
fi


%preun
%systemd_preun magellan

%postun
%systemd_postun_with_restart magellan

%files
%{prefix}
%exclude %{prefix}/test
%doc %{prefix}/api-definition.*
%attr(640, root, %{daemon_user}) %config(noreplace) %{prefix}/magellan.conf
%{_unitdir}/magellan.service
%{_sysconfdir}/sysconfig/magellan
%dir %attr(750, %{daemon_user}, root) %{data_dir}
%{backup_modules}

%files test
%{prefix}/test


%clean
rm -rf %{buildroot}


%changelog
* Fri Oct 29 2021 Aksel Sjögren <asjogren@itrsgroup.com>
- Install op5-backup module.
* Fri Jun  4 2021 Aksel Sjögren <asjogren@itrsgroup.com>
- Install magellan into prefix in buildtime to replace creating virtualenv in
  post script.
* Mon May 31 2021 Aksel Sjögren <asjogren@itrsgroup.com>
- Obsolete previous noarch releases of the package.
* Wed May 12 2021 Aksel Sjögren <asjogren@itrsgroup.com>
- Stop systemd service before removing application files.
* Wed Mar 31 2021 Aksel Sjögren <asjogren@itrsgroup.com>
- Add test subpackage with pre-installed test tools.
- Deploy Python wheels as individual files.
* Fri Feb  5 2021 Aksel Sjögren <asjogren@itrsgroup.com>
- Remove sudo rules.
* Fri Jan 29 2021 Aksel Sjögren <asjogren@itrsgroup.com>
- Changed license to OP5 EULA
* Thu Jan 28 2021 Petter Nyström <pnystrom@itrsgroup.com>
- Remove support for Python3.4
* Wed Jan 27 2021 Jacob Hansen <jhansen@itrsgroup.com>
- Changed license to GPLv3
- Changed URL to itrsgroup.com
* Wed Dec  2 2020 Aksel Sjögren <asjogren@itrsgroup.com>
- Remove old/redundant DB user on upgrades.
- Restrict permissions on files with DB credentials.
- Remove support for EL6.
* Wed Nov 11 2020 Aksel Sjögren <asjogren@itrsgroup.com> - 2020.j
- Move data directory from install path to /var/opt/magellan.
- Change package user to magellan.
- Add sudo rules to run nmap.
* Fri Feb 8 2019 Petter Nyström <pnystrom@op5.com>
- Package dependencies in install step to enable offline installation
* Tue Dec 4 2018 Petter Nyström <pnystrom@op5.com>
- Add the api-definition file
* Mon Nov 26 2018 Petter Nyström <pnystrom@op5.com>
- Adapt the install and start process on EL7 to best practices
* Thu May 3 2018 Petter Nyström <pnystrom@op5.com>
- Add EL6 support
* Thu Sep 28 2017 Mattias Ryrlén <mattiasr@op5.com>
- Initial build
